@@ -4,7 +4,7 @@
 #'
 #' @param txinrwd Whether active treatment is available in RWD (TRUE/FALSE). If FALSE, only the control arm of the RCT will be augmented with external data.
 #' @param data The dataset
-#' @param study Character name of variable indicating study participation (e.g. "S"). This variable should take a value of 1 for the RCT and should take other values for the other study(ies). Note that the code is currently set up only to handle two studies, but may be expanded to handle multiple studies in the future.
+#' @param study Character name of variable indicating study participation (e.g. "S"). This variable should take a value of 1 for the RCT and should take a value of 0 for the RWD. Note that the code is currently set up only to handle two studies, but may be expanded to handle multiple studies in the future.
 #' @param covariates Vector of character names of covariates to be adjusted for (e.g. c("W1", "W2"))
 #' @param treatment_var Character name of treatment variable (e.g. "A")
 #' @param treatment Value of treatment variable that corresponds to the active treatment (e.g. "DrugName" or 1). All other values of the treatment variable are assumed to be control.
@@ -12,10 +12,12 @@
 #' @param NCO Character name of negative control outcome variable (e.g. "nco") or NULL if no NCO available. If the NCO is a binary variable subject to censoring, censored observations should either be coded as NA or should be coded as 0 and a missingness indicator should be included (see parameter Delta_NCO below).
 #' @param Delta Character name of a variable that is 0 if an observation was censored (missing outcome) and 1 otherwise. Missing outcomes may also be coded as NA, in which case a Delta variable will be added internally. If no missing outcomes, set Delta=NULL.
 #' @param Delta_NCO Character name of a variable that is 0 if the value of NCO is missing and 1 otherwise. Missing NCOs may also be coded as NA, in which case a Delta_NCO variable will be added internally. If no missing NCO or no NCO, set Delta_NCO=NULL.
+#' @param id ID variable for the independent unit
 #' @param pRCT The probability of randomization to treatment in the RCT
 #' @param V Number of cross-validation folds (default 10)
 #' @param Q.SL.library Candidate algorithms for SuperLearner estimation of outcome regressions
-#' @param d.SL.library Candidate algorithms for SuperLearner estimation of missingness mechanism
+#' @param d.SL.library.RCT Candidate algorithms for SuperLearner estimation of missingness mechanism for RCT-only
+#' @param d.SL.library.RWD Candidate algorithms for SuperLearner estimation of missingness mechanism for RCT+RWD
 #' @param g.SL.library Candidate algorithms for SuperLearner estimation of treatment mechanism for combined RCT/RWD analysis
 #' @param Q.discreteSL Should a discrete SuperLearner be used for estimation of outcome regressions? (TRUE/FALSE)
 #' @param d.discreteSL Should a discrete SuperLearner be used for estimation of missingness mechanism? (TRUE/FALSE)
@@ -23,10 +25,12 @@
 #' @param family Either "binomial" for binary outcomes or "gaussian" for continuous outcomes
 #' @param family_nco Family for negative control outcome
 #' @param fluctuation 'logistic' (default for binary and continuous outcomes), or 'linear' describing fluctuation for targeted maximum likelihood estimation (TMLE) updating. If 'logistic' with a continuous outcome, outcomes are scaled to (0,1) for TMLE targeting and then returned to the original scale for parameter estimation.
-#' @param comparisons A list of the values of the study variable that you would like to compare. For example, if you have an RCT labeled S=1 and RWD labeled S=2, you would use comparisons = list(c(1),c(1,2)) to compare RCT only to RCT + RWD. The first element of comparisons must be c(1) for the RCT only.
+#' @param comparisons A list of the values of the study variable that you would like to compare. For example, if you have an RCT labeled S=1 and RWD labeled S=0, you would use comparisons = list(c(1),c(1,0)) to compare RCT only to RCT + RWD. The first element of comparisons must be c(1) for the RCT only.
 #' @param adjustnco Should we adjust for the NCO as a proxy of bias in the estimation of the ATE of A on Y? (TRUE/FALSE). Default is FALSE.
 #' @param target.gwt As in the tmle R package (Gruber & van der Laan, 2012), if target.gwt is TRUE, the treatment mechanism is moved from the denominator of the clever covariate to the weight when fitting the coefficient for TMLE updating. Default TRUE.
 #' @param bounds Optional bounds for truncation of the denominator of the clever covariate. The default is c(5/sqrt(n)/log(n),1).
+#' @param cvControl A list of parameters to control the cross-validation process for the SuperLearners. See ?SuperLearner for more details.
+#' @param MCsamp Number of Monte Carlo samples from the estimated limit distribution to use to estimate quantile-based confidence intervals. Default 1000.
 #'
 #' @importFrom origami make_folds
 #' @importFrom origami folds_vfold
@@ -67,60 +71,68 @@
 #' \donttest{data(wash)
 #' #For unbiased external controls, use:
 #' dat <- wash[which(wash$study %in% c(1,2)),]
+#' dat$study[which(dat$study==2)]<-0
 #' set.seed(2022)
 #' results_rwd1 <- ES.cvtmle(txinrwd=TRUE,
 #'                           data=dat, study="study",
 #'                           covariates=c("aged", "sex", "momedu", "hfiacat"),
-#'                           treatment_var="treatment", treatment=1,
+#'                           treatment_var="intervention", treatment=1,
 #'                           outcome="laz", NCO="Nlt18scale",
 #'                           Delta=NULL, Delta_NCO=NULL,
 #'                           pRCT=0.5, V=5, Q.SL.library=c("SL.glm"),
 #'                           g.SL.library=c("SL.glm"), Q.discreteSL=TRUE, g.discreteSL=TRUE,
 #'                           family="gaussian", family_nco="gaussian", fluctuation = "logistic",
-#'                           comparisons = list(c(1),c(1,2)), adjustnco = FALSE, target.gwt = TRUE)
+#'                           comparisons = list(c(1),c(1,0)), adjustnco = FALSE, target.gwt = TRUE)
 #' print.EScvtmle(results_rwd1)
 #' }
 #'
 #' @export
 
-ES.cvtmle <- function(txinrwd, data, study, covariates, treatment_var, treatment, outcome, NCO=NULL, Delta=NULL, Delta_NCO=NULL, pRCT, V=10, Q.SL.library, d.SL.library, g.SL.library, Q.discreteSL, d.discreteSL, g.discreteSL, family, family_nco, fluctuation = "logistic", comparisons = list(c(1),c(1,2)), adjustnco = FALSE, target.gwt = TRUE, bounds=NULL){
+ES.cvtmle <- function(txinrwd, data, study, covariates, treatment_var, treatment, outcome, NCO=NULL, Delta=NULL, Delta_NCO=NULL, id=NULL, pRCT, V=10, Q.SL.library, d.SL.library.RCT, d.SL.library.RWD, g.SL.library, Q.discreteSL, d.discreteSL, g.discreteSL, family, family_nco, fluctuation = "logistic", comparisons = list(c(1),c(1,0)), adjustnco = FALSE, target.gwt = TRUE, bounds=NULL, cvControl = list(), MCsamp=1000){
 
   if (length(comparisons)>2) stop("Package currently compares two experiments. Check back for updates to compare multiple experiments.")
 
+  if (any((data$study %in% c(0,1))==FALSE)) stop("Package currently considers two studies, where study=1 is an RCT, and study=0 is a real-world dataset.")
+
   if (comparisons[[1]]!=1) stop("First comparison should be c(1) (ie compare to RCT only).")
 
-  data <- preprocess(txinrwd, data, study, covariates, treatment_var, treatment, outcome, NCO, Delta, Delta_NCO, adjustnco)
-  if("Delta" %in% colnames(data)){
+  data <- preprocess(txinrwd, data, study, covariates, treatment_var, treatment, outcome, NCO, Delta, Delta_NCO, id, adjustnco)
+  if(any(data$Delta==0)){
     Delta = "Delta"
   }
 
-  if("NCO_delta" %in% colnames(data)){
+  if(any(data$NCO_delta==0)){
     Delta_NCO = "NCO_delta"
   }
 
   #Create cross-validation folds that preserve proportion of RCT (if txinwrd=TRUE) or of RCT controls (if txinrwd=FALSE) as well as proportion of outcomes in validation sets
-  ids <- data$S
 
-  if(txinrwd==TRUE){
-    if(family=="binomial"){
-      ids[which(data$S==1 & data$Y==1)] <- 0
+    ids <- data$S
+
+    if(txinrwd==TRUE){
+      if(family=="binomial"){
+        ids[which(data$S==1 & data$Y==1)] <- 0
+      } else {
+        ids[which(data$S==1)] <- 0
+      }
     } else {
-      ids[which(data$S==1)] <- 0
+      if(family=="binomial"){
+        ids[which(data$S==1 & data$A==0 & data$Y==1)] <- 0
+      } else {
+        ids[which(data$S==1 & data$A==0)] <- 0
+      }
     }
-  } else {
-    if(family=="binomial"){
-      ids[which(data$S==1 & data$A==0 & data$Y==1)] <- 0
-    } else {
-      ids[which(data$S==1 & data$A==0)] <- 0
-    }
-  }
+
+    folds <- make_folds(data, fold_fun = folds_vfold, V=V, cluster_ids = data$id, strata_ids = ids)
 
-  folds <- make_folds(data, fold_fun = folds_vfold, V=V, strata_ids = ids)
   data$v <- rep(NA, nrow(data))
   for(v in 1:V){
     data$v[folds[[v]]$validation_set]<-v
   }
 
+  #number of independent units
+  n.id <- length(unique(data$id))
+
   results <- list()
   selector <- list()
   valid_initial <- list()
@@ -134,8 +146,8 @@ ES.cvtmle <- function(txinrwd, data, study, covariates, treatment_var, treatment
   proportionselected$ncobias <- list()
 
   var_ay <- vector()
-  EICpsipound <- matrix(0, nrow=nrow(data), ncol=length(comparisons)*V)
-  EICnco <- matrix(0, nrow=nrow(data), ncol=length(comparisons)*V)
+  EICpsipound <- matrix(0, nrow=n.id, ncol=length(comparisons)*V)
+  EICnco <- matrix(0, nrow=n.id, ncol=length(comparisons)*V)
 
   bias <- list()
   bias_nco <- list()
@@ -154,12 +166,12 @@ ES.cvtmle <- function(txinrwd, data, study, covariates, treatment_var, treatment
     #define training set
     train <- data[sort(folds[[v]]$training_set),]
 
-    selector[[v]] <- apply_selector_func(txinrwd, train, data, Q.SL.library, d.SL.library, g.SL.library, pRCT, family, family_nco, fluctuation, NCO, Delta, Delta_NCO, adjustnco, target.gwt, Q.discreteSL, d.discreteSL, g.discreteSL, comparisons, bounds)
+    selector[[v]] <- apply_selector_func(txinrwd, train, data, Q.SL.library, d.SL.library.RCT, d.SL.library.RWD, g.SL.library, pRCT, family, family_nco, fluctuation, NCO, Delta, Delta_NCO, adjustnco, target.gwt, Q.discreteSL, d.discreteSL, g.discreteSL, comparisons, bounds, cvControl)
 
     if(txinrwd==TRUE){
-      bvt[[v]] <- bvt_txinrwd(v, selector, NCO, comparisons, train, data, fluctuation, family)
+      bvt[[v]] <- bvt_txinrwd(v, selector, NCO, comparisons, train, data, fluctuation, family, n.id)
     } else {
-      bvt[[v]] <- bvt_notxinrwd(v, selector, NCO, comparisons, train, data, fluctuation, family)
+      bvt[[v]] <- bvt_notxinrwd(v, selector, NCO, comparisons, train, data, fluctuation, family, n.id)
     }
 
     lambdatilde$b2v[[v]] <- comparisons[[which(bvt[[v]]$b2v==min(bvt[[v]]$b2v))]]
@@ -190,7 +202,7 @@ ES.cvtmle <- function(txinrwd, data, study, covariates, treatment_var, treatment
   results$foldATEs$ncobias <- vector()
 
   #estimate components of limit distribution
-  limitdist <- limitdistvar(V, valid_initial, data, folds, family, fluctuation, Delta, pRCT, target.gwt, comparisons, bounds)
+  limitdist <- limitdistvar(V, valid_initial, data, folds, family, fluctuation, Delta, pRCT, target.gwt, comparisons, bounds, n.id)
 
   results$g <- list()
   for(s in 1:length(comparisons)){
@@ -217,7 +229,7 @@ ES.cvtmle <- function(txinrwd, data, study, covariates, treatment_var, treatment
   }
 
   #sample from limit distribution
-  limitdistsamp <- limitdist_sample(V, bvt, NCO, EICpsipound, EICnco, var_ay, limitdist, data, comparisons)
+  limitdistsamp <- limitdist_sample(V, bvt, NCO, EICpsipound, EICnco, var_ay, limitdist, n.id, comparisons, MCsamp)
 
   results$CI$b2v <- list()
   results$CI$ncobias <- list()
