@@ -1,7 +1,7 @@
 #' @importFrom SuperLearner SuperLearner
 #' @importFrom stats predict
 #Function for training initial estimators for the outcome, treatment mechanism, and study selection mechanism regressions for RCT with or without real-world data (when active treatment is available in real-world data)
-selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library, g.SL.library, pRCT, family, family_nco, fluctuation = "logistic", NCO=NULL, Delta=NULL, Delta_NCO = NULL, adjustnco, target.gwt, Q.discreteSL, d.discreteSL, g.discreteSL, bounds){
+selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library.RCT, d.SL.library.RWD, g.SL.library, pRCT, family, family_nco, fluctuation = "logistic", NCO=NULL, Delta=NULL, Delta_NCO = NULL, adjustnco, target.gwt, Q.discreteSL, d.discreteSL, g.discreteSL, bounds=NULL, cvControl=list()){
 
   #Train regressions for different experiments
   if(any(train_s$S==0)){
@@ -12,9 +12,9 @@ selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library, g.SL.
     }
 
     if(adjustnco == FALSE){
-      X <- train_s[,which((colnames(train_s) %in% c("Y", "nco", "NCO_delta", "v"))==FALSE)]
+      X <- train_s[,which((colnames(train_s) %in% c("Y", "nco", "NCO_delta", "v", "id"))==FALSE)]
     } else {
-      X <- train_s[,which((colnames(train_s) %in% c("Y", "NCO_delta", "v"))==FALSE)]
+      X <- train_s[,which((colnames(train_s) %in% c("Y", "NCO_delta", "v", "id"))==FALSE)]
     }
 
 
@@ -23,25 +23,21 @@ selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library, g.SL.
     X0$A <- 0
     X1$A <- 1
 
-    if(is.null(Delta)==FALSE){
-      Ynomiss <- Y[which(X$Delta==1)]
-      Xnomiss <- X[which(X$Delta==1),]
-      Xnomiss <- Xnomiss[ , -which(colnames(Xnomiss) %in% c("Delta"))]
-    } else {
-      Ynomiss <- Y
-      Xnomiss <- X
-    }
+    Ynomiss <- Y[which(X$Delta==1)]
+    Xnomiss <- X[which(X$Delta==1),]
+    Xnomiss <- Xnomiss[ , -which(colnames(Xnomiss) %in% c("Delta"))]
+
 
     # call Super Learner for estimation of QbarAW
     if(fluctuation == "logistic"){
-      QbarSL<- suppressWarnings(SuperLearner(Y=Ynomiss, X=Xnomiss, SL.library=Q.SL.library, family="binomial", control = list(saveFitLibrary=TRUE)))
+      QbarSL<- suppressWarnings(SuperLearner(Y=Ynomiss, X=Xnomiss, SL.library=Q.SL.library, family="binomial", control = list(saveFitLibrary=TRUE), cvControl = cvControl, id=train_s$id[which(X$Delta==1)]))
       if(Q.discreteSL==TRUE){
         keepAlg <- which.min(QbarSL$cvRisk)
         QbarSL <- QbarSL$fitLibrary[[keepAlg]]
       }
 
     } else {
-      QbarSL<- SuperLearner(Y=Ynomiss, X=Xnomiss, SL.library=Q.SL.library, family=family)
+      QbarSL<- suppressWarnings(SuperLearner(Y=Ynomiss, X=Xnomiss, SL.library=Q.SL.library, family=family, cvControl = cvControl, id=train_s$id[which(X$Delta==1)]))
       if(Q.discreteSL==TRUE){
         keepAlg <- which.min(QbarSL$cvRisk)
         QbarSL <- QbarSL$fitLibrary[[keepAlg]]
@@ -63,7 +59,7 @@ selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library, g.SL.
 
     dHat1W <- list()
     if(is.null(Delta)==FALSE){
-      DbarSL<- SuperLearner(Y=X$Delta, X=X[ , -which(colnames(X) %in% c("Delta"))], SL.library=d.SL.library, family="binomial", control = list(saveFitLibrary=TRUE))
+      DbarSL<- suppressWarnings(SuperLearner(Y=X$Delta, X=X[ , -which(colnames(X) %in% c("Delta"))], SL.library=d.SL.library.RWD, family="binomial", control = list(saveFitLibrary=TRUE), cvControl = cvControl, id=train_s$id))
       if(d.discreteSL==TRUE){
         keepAlg <- which.min(DbarSL$cvRisk)
         DbarSL <- DbarSL$fitLibrary[[keepAlg]]
@@ -79,7 +75,7 @@ selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library, g.SL.
     }
 
     # Estimate the exposure mechanism g(A|W)
-    gHatSL<- SuperLearner(Y=X$A, X=X[ , -which(colnames(X) %in% c("A","Delta"))], SL.library=g.SL.library, family="binomial", control = list(saveFitLibrary=TRUE))
+    gHatSL<- suppressWarnings(SuperLearner(Y=X$A, X=X[ , -which(colnames(X) %in% c("A","Delta"))], SL.library=g.SL.library, family="binomial", control = list(saveFitLibrary=TRUE), cvControl = cvControl, id=train_s$id))
     if(g.discreteSL==TRUE){
       keepAlg <- which.min(gHatSL$cvRisk)
       gHatSL <- gHatSL$fitLibrary[[keepAlg]]
@@ -138,7 +134,7 @@ selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library, g.SL.
     }
 
     # Estimate the trial participation mechanism g(S|A,W)
-    gSHatSL<- SuperLearner(Y=X$S, X=X[ , -which(names(X) %in% c("S","A","Delta"))], SL.library=g.SL.library, family="binomial", control = list(saveFitLibrary=TRUE))
+    gSHatSL<- suppressWarnings(SuperLearner(Y=X$S, X=X[ , -which(names(X) %in% c("S","A","Delta"))], SL.library=g.SL.library, family="binomial", control = list(saveFitLibrary=TRUE), cvControl = cvControl, id=train_s$id))
     if(g.discreteSL==TRUE){
       keepAlg <- which.min(gSHatSL$cvRisk)
       gSHatSL <- gSHatSL$fitLibrary[[keepAlg]]
@@ -199,29 +195,25 @@ selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library, g.SL.
         train_s_nco <- train_s$nco
       }
 
-      X <- train_s[,which((colnames(train_s) %in% c("Y", "nco", "Delta", "v"))==FALSE)]
+      X <- train_s[,which((colnames(train_s) %in% c("Y", "nco", "Delta", "v", "id"))==FALSE)]
       X0 <- X1 <- X
       X0$A <- 0
       X1$A <- 1
 
-      if(is.null(Delta_NCO)==FALSE){
-        NCOnomiss <- train_s_nco[which(X$NCO_delta==1)]
-        Xnomiss <- X[which(X$NCO_delta==1),]
-        Xnomiss <- Xnomiss[ , -which(colnames(Xnomiss) %in% c("NCO_delta"))]
-      } else {
-        NCOnomiss <- train_s_nco
-        Xnomiss <- X
-        X$NCO_delta <- rep(1, nrow(X))
-      }
+
+      NCOnomiss <- train_s_nco[which(X$NCO_delta==1)]
+      Xnomiss <- X[which(X$NCO_delta==1),]
+      Xnomiss <- Xnomiss[ , -which(colnames(Xnomiss) %in% c("NCO_delta"))]
+
 
       if(fluctuation == "logistic"){
-        NCObarSL<- suppressWarnings(SuperLearner(Y=NCOnomiss, X=Xnomiss, SL.library=Q.SL.library, family="binomial", control = list(saveFitLibrary=TRUE)))
+        NCObarSL<- suppressWarnings(SuperLearner(Y=NCOnomiss, X=Xnomiss, SL.library=Q.SL.library, family="binomial", control = list(saveFitLibrary=TRUE), cvControl = cvControl, id=train_s$id[which(X$NCO_delta==1)]))
         if(Q.discreteSL==TRUE){
           keepAlg <- which.min(NCObarSL$cvRisk)
           NCObarSL <- NCObarSL$fitLibrary[[keepAlg]]
         }
       } else {
-        NCObarSL<- SuperLearner(Y=NCOnomiss, X=Xnomiss, SL.library=Q.SL.library, family=family, control = list(saveFitLibrary=TRUE))
+        NCObarSL<- suppressWarnings(SuperLearner(Y=NCOnomiss, X=Xnomiss, SL.library=Q.SL.library, family=family, control = list(saveFitLibrary=TRUE), cvControl = cvControl, id=train_s$id[which(X$NCO_delta==1)]))
         if(Q.discreteSL==TRUE){
           keepAlg <- which.min(NCObarSL$cvRisk)
           NCObarSL <- NCObarSL$fitLibrary[[keepAlg]]
@@ -239,7 +231,7 @@ selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library, g.SL.
       }
 
       # Estimate the exposure mechanism g(A|W)
-      gHatSLnco<- SuperLearner(Y=X$A, X=X[ , -which(colnames(X) %in% c("A","NCO_delta"))], SL.library=g.SL.library, family="binomial", control = list(saveFitLibrary=TRUE))
+      gHatSLnco<- suppressWarnings(SuperLearner(Y=X$A, X=X[ , -which(colnames(X) %in% c("A","NCO_delta"))], SL.library=g.SL.library, family="binomial", control = list(saveFitLibrary=TRUE), cvControl = cvControl, id=train_s$id))
       if(g.discreteSL==TRUE){
         keepAlg <- which.min(gHatSLnco$cvRisk)
         gHatSLnco <- gHatSLnco$fitLibrary[[keepAlg]]
@@ -253,7 +245,7 @@ selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library, g.SL.
 
       dHat1Wnco <- list()
       if(is.null(Delta_NCO)==FALSE){
-        DbarSLnco<- SuperLearner(Y=X$NCO_delta, X=X[ , -which(colnames(X) %in% c("NCO_delta"))], SL.library=d.SL.library, family="binomial", control = list(saveFitLibrary=TRUE))
+        DbarSLnco<- suppressWarnings(SuperLearner(Y=X$NCO_delta, X=X[ , -which(colnames(X) %in% c("NCO_delta"))], SL.library=d.SL.library.RWD, family="binomial", control = list(saveFitLibrary=TRUE), cvControl = cvControl, id=train_s$id))
         if(d.discreteSL==TRUE){
           keepAlg <- which.min(DbarSLnco$cvRisk)
           DbarSLnco <- DbarSLnco$fitLibrary[[keepAlg]]
@@ -311,9 +303,9 @@ selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library, g.SL.
 
     #run tmle for E[E[Y|W,A=0]]
     if(adjustnco == FALSE){
-      X <- train_s[,which((colnames(train_s) %in% c("S","Y", "nco", "NCO_delta", "v"))==FALSE)]
+      X <- train_s[,which((colnames(train_s) %in% c("S","Y", "nco", "NCO_delta", "v", "id"))==FALSE)]
     } else {
-      X <- train_s[,which((colnames(train_s) %in% c("S","Y", "NCO_delta", "v"))==FALSE)]
+      X <- train_s[,which((colnames(train_s) %in% c("S","Y", "NCO_delta", "v", "id"))==FALSE)]
     }
 
 
@@ -322,24 +314,19 @@ selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library, g.SL.
     X0$A <- 0
     X1$A <- 1
 
-    if(is.null(Delta)==FALSE){
-      Ynomiss <- Y[which(X$Delta==1)]
-      Xnomiss <- X[which(X$Delta==1),]
-      Xnomiss <- Xnomiss[ , -which(colnames(Xnomiss) %in% c("Delta"))]
-    } else {
-      Ynomiss <- Y
-      Xnomiss <- X
-    }
+    Ynomiss <- Y[which(X$Delta==1)]
+    Xnomiss <- X[which(X$Delta==1),]
+    Xnomiss <- Xnomiss[ , -which(colnames(Xnomiss) %in% c("Delta"))]
 
     # call Super Learner for estimation of QbarAW
     if(fluctuation == "logistic"){
-      QbarSL<- suppressWarnings(SuperLearner(Y=Ynomiss, X=Xnomiss, SL.library=Q.SL.library, family="binomial", control = list(saveFitLibrary=TRUE)))
+      QbarSL<- suppressWarnings(SuperLearner(Y=Ynomiss, X=Xnomiss, SL.library=Q.SL.library, family="binomial", control = list(saveFitLibrary=TRUE), cvControl = cvControl, id=train_s$id[which(X$Delta==1)]))
       if(Q.discreteSL==TRUE){
         keepAlg <- which.min(QbarSL$cvRisk)
         QbarSL <- QbarSL$fitLibrary[[keepAlg]]
       }
     } else {
-      QbarSL<- SuperLearner(Y=Ynomiss, X=Xnomiss, SL.library=Q.SL.library, family=family, control = list(saveFitLibrary=TRUE))
+      QbarSL<- suppressWarnings(SuperLearner(Y=Ynomiss, X=Xnomiss, SL.library=Q.SL.library, family=family, control = list(saveFitLibrary=TRUE), cvControl = cvControl, id=train_s$id[which(X$Delta==1)]))
       if(Q.discreteSL==TRUE){
         keepAlg <- which.min(QbarSL$cvRisk)
         QbarSL <- QbarSL$fitLibrary[[keepAlg]]
@@ -362,7 +349,7 @@ selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library, g.SL.
 
     dHat1W <- list()
     if(is.null(Delta)==FALSE){
-      DbarSL<- SuperLearner(Y=X$Delta, X=X[ , -which(colnames(X) %in% c("Delta"))], SL.library=d.SL.library, family="binomial", control = list(saveFitLibrary=TRUE))
+      DbarSL<- suppressWarnings(SuperLearner(Y=X$Delta, X=X[ , -which(colnames(X) %in% c("Delta"))], SL.library=d.SL.library.RCT, family="binomial", control = list(saveFitLibrary=TRUE), cvControl = cvControl, id=train_s$id))
       if(d.discreteSL==TRUE){
         keepAlg <- which.min(DbarSL$cvRisk)
         DbarSL <- DbarSL$fitLibrary[[keepAlg]]
@@ -424,28 +411,24 @@ selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library, g.SL.
         train_s_nco <- train_s$nco
       }
 
-      X <- train_s[,which((colnames(train_s) %in% c("S","Y", "nco", "Delta", "v"))==FALSE)]
+      X <- train_s[,which((colnames(train_s) %in% c("S","Y", "nco", "Delta", "v", "id"))==FALSE)]
       X0 <- X1 <- X
       X0$A <- 0
       X1$A <- 1
 
-      if(is.null(Delta_NCO)==FALSE){
-        NCOnomiss <- train_s_nco[which(X$NCO_delta==1)]
-        Xnomiss <- X[which(X$NCO_delta==1),]
-        Xnomiss <- Xnomiss[ , -which(colnames(Xnomiss) %in% c("NCO_delta"))]
-      } else {
-        NCOnomiss <- train_s_nco
-        Xnomiss <- X
-      }
+      NCOnomiss <- train_s_nco[which(X$NCO_delta==1)]
+      Xnomiss <- X[which(X$NCO_delta==1),]
+      Xnomiss <- Xnomiss[ , -which(colnames(Xnomiss) %in% c("NCO_delta"))]
+
 
       if(fluctuation == "logistic"){
-        NCObarSL<- suppressWarnings(SuperLearner(Y=NCOnomiss, X=Xnomiss, SL.library=Q.SL.library, family="binomial", control = list(saveFitLibrary=TRUE)))
+        NCObarSL<- suppressWarnings(SuperLearner(Y=NCOnomiss, X=Xnomiss, SL.library=Q.SL.library, family="binomial", control = list(saveFitLibrary=TRUE), cvControl = cvControl, id=train_s$id[which(X$NCO_delta==1)]))
         if(Q.discreteSL==TRUE){
           keepAlg <- which.min(NCObarSL$cvRisk)
           NCObarSL <- NCObarSL$fitLibrary[[keepAlg]]
         }
       } else {
-        NCObarSL<- SuperLearner(Y=NCOnomiss, X=Xnomiss, SL.library=Q.SL.library, family=family, control = list(saveFitLibrary=TRUE))
+        NCObarSL<- suppressWarnings(SuperLearner(Y=NCOnomiss, X=Xnomiss, SL.library=Q.SL.library, family=family, control = list(saveFitLibrary=TRUE), cvControl = cvControl, id=train_s$id[which(X$NCO_delta==1)]))
         if(Q.discreteSL==TRUE){
           keepAlg <- which.min(NCObarSL$cvRisk)
           NCObarSL <- NCObarSL$fitLibrary[[keepAlg]]
@@ -468,7 +451,7 @@ selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library, g.SL.
 
       dHat1Wnco <- list()
       if(is.null(Delta_NCO)==FALSE){
-        DbarSLnco<- SuperLearner(Y=X$NCO_delta, X=X[ , -which(colnames(X) %in% c("NCO_delta"))], SL.library=d.SL.library, family="binomial", control = list(saveFitLibrary=TRUE))
+        DbarSLnco<- suppressWarnings(SuperLearner(Y=X$NCO_delta, X=X[ , -which(colnames(X) %in% c("NCO_delta"))], SL.library=d.SL.library.RCT, family="binomial", control = list(saveFitLibrary=TRUE), cvControl = cvControl, id=train_s$id))
         if(d.discreteSL==TRUE){
           keepAlg <- which.min(DbarSLnco$cvRisk)
           DbarSLnco <- DbarSLnco$fitLibrary[[keepAlg]]
@@ -529,7 +512,7 @@ selector_func_txrwd <- function(train_s, data, Q.SL.library, d.SL.library, g.SL.
 #' @importFrom stats plogis
 #' @importFrom stats qlogis
 #Function to estimate the bias-variance tradeoff using the experiment-selection set for each fold when active treatment is available in the real-world data
-bvt_txinrwd <- function(v, selector, NCO, comparisons, train, data, fluctuation, family){
+bvt_txinrwd <- function(v, selector, NCO, comparisons, train, data, fluctuation, family, n.id){
   out <- list()
 
   out$b2v <- vector()
@@ -537,20 +520,19 @@ bvt_txinrwd <- function(v, selector, NCO, comparisons, train, data, fluctuation,
   out$var <- vector()
   out$EIClambdav <- list()
 
+  datid <- data[which(duplicated(data$id)==FALSE),]
+
   if(is.null(NCO)==FALSE){
     out$addncobias <- vector()
     out$bias_nco <- vector()
   }
 
-  out$EICpsipound <- matrix(0, nrow=nrow(data), ncol=length(comparisons))
-  out$EICnco <- matrix(0, nrow=nrow(data), ncol=length(comparisons))
+  out$EICpsipound <- matrix(0, nrow=n.id, ncol=length(comparisons))
+  out$EICnco <- matrix(0, nrow=n.id, ncol=length(comparisons))
 
 
   for(s in 1:length(comparisons)){
     train_s <- train[which(train$S %in% comparisons[[s]]),]
-    if(is.null(train_s$Delta)){
-      train_s$Delta <- rep(1, nrow(train_s))
-    }
 
     #tmle
     if(fluctuation == "logistic"){
@@ -581,7 +563,11 @@ bvt_txinrwd <- function(v, selector, NCO, comparisons, train, data, fluctuation,
       trainsY <- selector[[v]][[s]]$Y
     }
 
-    out$EIClambdav[[s]] <- (selector[[v]][[s]]$wt*(selector[[v]][[s]]$H.AW1 + selector[[v]][[s]]$H.AW0)*(trainsY - QbarAW.star) + Qbar1W.star - Qbar0W.star - mean(Qbar1W.star - Qbar0W.star))
+    #EIC
+    EIClambdav_s <- (selector[[v]][[s]]$wt*(selector[[v]][[s]]$H.AW1 + selector[[v]][[s]]$H.AW0)*(trainsY - QbarAW.star) + Qbar1W.star - Qbar0W.star - mean(Qbar1W.star - Qbar0W.star))
+    #account for dependent units
+    #take mean of EIC observations by trains_s$id (confirmed taking mean correctly)
+    out$EIClambdav[[s]] <- as.vector(by(EIClambdav_s, train_s$id, mean))
 
     if(s==1){
       QbarS1W.star <- Qbar1W.star
@@ -614,19 +600,17 @@ bvt_txinrwd <- function(v, selector, NCO, comparisons, train, data, fluctuation,
     }
 
     if(s>1){
-      out$EICpsipound[which(data$v!=v & data$S %in% comparisons[[s]]),s] <- (selector[[v]][[s]]$wt*(selector[[v]][[s]]$H.AW1 + selector[[v]][[s]]$H.AW0)*(trainsY - QbarAW.star) + Qbar1W.star - Qbar0W.star - mean(Qbar1W.star - Qbar0W.star) - (selector[[v]][[s]]$wt_s*(selector[[v]][[s]]$H.SAW1 + selector[[v]][[s]]$H.SAW0)*(trainsY - QbarSAW.star) + QbarS1W.star - QbarS0W.star - mean(QbarS1W.star - QbarS0W.star)))/(length(which(train$S %in% comparisons[[s]]))/nrow(data))
+      EICpsipound_s <- (selector[[v]][[s]]$wt*(selector[[v]][[s]]$H.AW1 + selector[[v]][[s]]$H.AW0)*(trainsY - QbarAW.star) + Qbar1W.star - Qbar0W.star - mean(Qbar1W.star - Qbar0W.star) - (selector[[v]][[s]]$wt_s*(selector[[v]][[s]]$H.SAW1 + selector[[v]][[s]]$H.SAW0)*(trainsY - QbarSAW.star) + QbarS1W.star - QbarS0W.star - mean(QbarS1W.star - QbarS0W.star)))
+      EICpsipound_s <- as.vector(by(EICpsipound_s, train_s$id, mean))
+      out$EICpsipound[which(datid$v!=v & datid$S %in% comparisons[[s]]),s] <- EICpsipound_s/(length(which(train$S %in% comparisons[[s]] & duplicated(train$id)==FALSE))/n.id)
     }
 
     out$bias[s] <- mean(Qbar1W.star - Qbar0W.star) - mean(QbarS1W.star - QbarS0W.star)
-    out$var[s] <- var(out$EIClambdav[[s]])/length(trainsY)
+    out$var[s] <- var(out$EIClambdav[[s]])/length(unique(train_s$id))
 
     #nco
     if(is.null(NCO)==FALSE){
 
-      if(is.null(train_s$NCO_delta)){
-        train_s$NCO_delta <- rep(1, nrow(train_s))
-      }
-
       if(fluctuation == "logistic"){
         logitUpdate<- glm(selector[[v]][[s]]$train_s_nco[which(train_s$NCO_delta==1)] ~ -1 + offset(qlogis(selector[[v]][[s]]$NCObarAW[which(train_s$NCO_delta==1)])) +  selector[[v]][[s]]$H.AWnco[which(train_s$NCO_delta==1)], family='quasibinomial', weights = selector[[v]][[s]]$wt_nco[which(train_s$NCO_delta==1)])
 
@@ -655,7 +639,10 @@ bvt_txinrwd <- function(v, selector, NCO, comparisons, train, data, fluctuation,
         train_s_nco <- selector[[v]][[s]]$train_s_nco
       }
 
-      out$EICnco[which(data$v!=v & data$S %in% comparisons[[s]]),s] <- (selector[[v]][[s]]$wt_nco*selector[[v]][[s]]$H.AWnco*(train_s_nco - NCObarAW.star) + NCObar1W.star - NCObar0W.star - mean(NCObar1W.star - NCObar0W.star))/(length(which(train$S %in% comparisons[[s]]))/nrow(data))
+      EICnco_s <- as.vector(selector[[v]][[s]]$wt_nco*selector[[v]][[s]]$H.AWnco*(train_s_nco - NCObarAW.star) + NCObar1W.star - NCObar0W.star - mean(NCObar1W.star - NCObar0W.star))
+      EICnco_s <- as.vector(by(EICnco_s, train_s$id, mean))
+
+      out$EICnco[which(datid$v!=v & datid$S %in% comparisons[[s]]),s] <- EICnco_s/(length(which(train$S %in% comparisons[[s]] & duplicated(train$id)==FALSE))/n.id)
 
       out$bias_nco[s] <- mean(NCObar1W.star - NCObar0W.star)
 
